Package: genetics
Title: Population Genetics
Version: 1.3.8.1.3
Date: 2012-11-26
Author: Gregory Warnes, with contributions from Gregor Gorjanc,
        Friedrich Leisch, and Michael Man.
Maintainer: Gregory Warnes <greg@warnes.net>
Depends: combinat, gdata, gtools, MASS, mvtnorm
Description: Classes and methods for handling genetic data. Includes
        classes to represent genotypes and haplotypes at single markers
        up to multiple markers on multiple chromosomes. Function
        include allele frequencies, flagging homo/heterozygotes,
        flagging carriers of certain alleles, estimating and testing
        for Hardy-Weinberg disequilibrium, estimating and testing for
        linkage disequilibrium, ...
biocViews: Genetics
License: GPL
Packaged: 2021-03-01 06:56:06 UTC; hornik
Repository: CRAN
Date/Publication: 2021-03-01 07:10:07 UTC
NeedsCompilation: no
Built: R 4.6.0; ; 2026-04-25 18:04:39 UTC; unix
